#![allow(dead_code)]
use nmea0183::{
    GGA,
    GLL,
    coords::{
        Latitude as NMEALatitude,
        Longitude as NMEALongitude,
        Hemisphere as NMEAHemisphere,
    },
};
use serde::Serialize;

#[derive(Debug, Serialize)]
pub enum Hemisphere {
    North,
    South,
    East,
    West,
}

#[derive(Debug, Serialize)]
pub struct Latitude {
    degrees: u8,
    minutes: u8,
    seconds: f32,
    hemisphere: Hemisphere,
}

#[derive(Debug, Serialize)]
pub struct Longitude {
    degrees: u8,
    minutes: u8,
    seconds: f32,
    hemisphere: Hemisphere,
}

impl From<NMEALatitude> for Latitude {
    fn from(lat: NMEALatitude) -> Self {
        Self {
            degrees: lat.degrees,
            minutes: lat.minutes,
            seconds: lat.seconds,
            hemisphere: lat.hemisphere.into(),
        }
    }
}

impl From<NMEALongitude> for Longitude {
    fn from(lon: NMEALongitude) -> Self {
        Self {
            degrees: lon.degrees,
            minutes: lon.minutes,
            seconds: lon.seconds,
            hemisphere: lon.hemisphere.into(),
        }
    }
}

impl From<NMEAHemisphere> for Hemisphere {
    fn from(hem: NMEAHemisphere) -> Self {
        match hem {
            NMEAHemisphere::North => Self::North,
            NMEAHemisphere::South => Self::South,
            NMEAHemisphere::East => Self::East,
            NMEAHemisphere::West => Self::West,
        }
    }
}

#[derive(Debug, Serialize)]
pub struct Solution {
    pub latitude: Latitude,
    pub longitude: Longitude,
    pub altitude: Option<f32>,
    pub speed: Option<f32>,
    pub direction: Option<f32>,
}

impl From<GGA> for Solution {
    fn from(gga: GGA) -> Self {
        Self {
            latitude: gga.latitude.into(),
            longitude: gga.longitude.into(),
            altitude: Some(gga.altitude.meters),
            speed: None,
            direction: None,
        }
    }
}

impl From<GLL> for Solution {
    fn from(gll: GLL) -> Self {
        Self {
            latitude: gll.latitude.into(),
            longitude: gll.longitude.into(),
            altitude: None,
            speed: None,
            direction: None,
        }
    }
}

pub enum Msg {
    Gps(Solution),
    Accelerometer(String),
}
